﻿//---------------------------------------------------------------------------
//
//   MQWidget
//
//          Copyright(C) 1999-2013, tetraface Inc.
//
//	   A class for accessing widgets to construct a graphical user interfaces 
//    such as buttons, check boxes, dialogs and so on.
//
//    　ボタン、チェックボックスやダイアログなどのGUIを構築するためのウィ
//    ジェットにアクセスするクラスです。
//
//---------------------------------------------------------------------------

#include <windows.h>
#include <map>
#include <assert.h>
#include "MQPlugin.h"
#include "MQBasePlugin.h"
#include "MQWidget.h"

static void *ExtractEventOption(void *option, const char *name);
static BOOL ExtractEventOptionBool(void *option, const char *name, BOOL defval);
static int ExtractEventOptionInt(void *option, const char *name, int defval);
static float ExtractEventOptionFloat(void *option, const char *name, float defval);


static std::map<int, MQWidgetBase*> s_WidgetIDMap;

//---------------------------------------------------------------------------
//  class MQWidgetBase
//---------------------------------------------------------------------------
MQWidgetBase::MQWidgetBase()
{
	m_ID = NullID;
	m_IDOwner = false;
}

MQWidgetBase::MQWidgetBase(int id)
{
	m_ID = id;
	m_IDOwner = false;

	if(m_ID != NullID){
		s_WidgetIDMap.insert(std::pair<int,MQWidgetBase*>(m_ID, this));
	}
}

MQWidgetBase::~MQWidgetBase()
{
	if(m_ID != NullID){
		s_WidgetIDMap.erase(m_ID);

		if(m_IDOwner){
			void *ptr[1];
			ptr[0] = NULL;
			MQWidget_Value(m_ID, MQWIDGET_DELETE, (void*)ptr);
		}

		m_ID = NullID;
		m_IDOwner = false;
	}
}

int MQWidgetBase::GetID() const
{
	return m_ID;
}

int MQWidgetBase::AddChild(MQWidgetBase *child)
{
	if(child == NULL) return -1;

	void *ptr[5];
	int child_id = child->GetID();
	int result = -1;

	ptr[0] = "child";
	ptr[1] = &child_id;
	ptr[2] = "result";
	ptr[3] = &result;
	ptr[4] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_ADD_CHILD, (void*)ptr);
	
	return result;
}

void MQWidgetBase::RemoveChild(MQWidgetBase *child)
{
	if(child == NULL) return;

	void *ptr[3];
	int child_id = child->GetID();
	int result = -1;

	ptr[0] = "child";
	ptr[1] = &child_id;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_REMOVE_CHILD, (void*)ptr);
}

bool MQWidgetBase::GetEnabled()
{
	bool value = false;
	void *ptr[3];
	ptr[0] = "enabled";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQWidgetBase::SetEnabled(bool value)
{
	void *ptr[3];
	ptr[0] = "enabled";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

bool MQWidgetBase::GetVisible()
{
	bool value = false;
	void *ptr[3];
	ptr[0] = "visible";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);
	return value;
}

void MQWidgetBase::SetVisible(bool value)
{
	void *ptr[3];
	ptr[0] = "visible";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

MQWidgetBase::LAYOUT_TYPE MQWidgetBase::GetHorzLayout()
{
	char *layout_str = NULL;
	void *ptr[3];
	ptr[0] = "horzlayout";
	ptr[1] = &layout_str;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	if(strcmp(layout_str, "auto") == 0) return LAYOUT_AUTO;
	if(strcmp(layout_str, "fixed") == 0) return LAYOUT_FIXED;
	if(strcmp(layout_str, "hintsize") == 0) return LAYOUT_HINTSIZE;
	if(strcmp(layout_str, "fill") == 0) return LAYOUT_FILL;
	if(strcmp(layout_str, "free") == 0) return LAYOUT_FREE;
	return LAYOUT_AUTO;
}

void MQWidgetBase::SetHorzLayout(LAYOUT_TYPE value)
{
	void *ptr[3];
	ptr[0] = "horzlayout";
	switch(value){
	default:
	case LAYOUT_AUTO: ptr[1] = "auto"; break;
	case LAYOUT_FIXED: ptr[1] = "fixed"; break;
	case LAYOUT_HINTSIZE: ptr[1] = "hintsize"; break;
	case LAYOUT_FILL: ptr[1] = "fill"; break;
	case LAYOUT_FREE: ptr[1] = "free"; break;
	}
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

MQWidgetBase::LAYOUT_TYPE MQWidgetBase::GetVertLayout()
{
	char *layout_str = NULL;
	void *ptr[3];
	ptr[0] = "vertlayout";
	ptr[1] = &layout_str;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	if(strcmp(layout_str, "auto") == 0) return LAYOUT_AUTO;
	if(strcmp(layout_str, "fixed") == 0) return LAYOUT_FIXED;
	if(strcmp(layout_str, "hintsize") == 0) return LAYOUT_HINTSIZE;
	if(strcmp(layout_str, "fill") == 0) return LAYOUT_FILL;
	if(strcmp(layout_str, "free") == 0) return LAYOUT_FREE;
	return LAYOUT_AUTO;
}

void MQWidgetBase::SetVertLayout(LAYOUT_TYPE value)
{
	void *ptr[3];
	ptr[0] = "vertlayout";
	switch(value){
	default:
	case LAYOUT_AUTO: ptr[1] = "auto"; break;
	case LAYOUT_FIXED: ptr[1] = "fixed"; break;
	case LAYOUT_HINTSIZE: ptr[1] = "hintsize"; break;
	case LAYOUT_FILL: ptr[1] = "fill"; break;
	case LAYOUT_FREE: ptr[1] = "free"; break;
	}
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

int MQWidgetBase::GetWidth()
{
	int value = 0;
	void *ptr[3];
	ptr[0] = "width";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQWidgetBase::SetWidth(int value)
{
#ifdef _DEBUG
	assert(dynamic_cast<MQWindow*>(this) != NULL || GetHorzLayout() == LAYOUT_FIXED);
#endif

	void *ptr[3];
	ptr[0] = "width";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

int MQWidgetBase::GetHeight()
{
	int value = 0;
	void *ptr[3];
	ptr[0] = "height";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQWidgetBase::SetHeight(int value)
{
#ifdef _DEBUG
	assert(dynamic_cast<MQWindow*>(this) != NULL || GetVertLayout() == LAYOUT_FIXED);
#endif

	void *ptr[3];
	ptr[0] = "height";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

double MQWidgetBase::GetFillRateX()
{
	double value = 0;
	void *ptr[3];
	ptr[0] = "fillrate_x";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQWidgetBase::SetFillRateX(double value)
{
	void *ptr[3];
	ptr[0] = "fillrate_x";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

double MQWidgetBase::GetFillRateY()
{
	double value = 0;
	void *ptr[3];
	ptr[0] = "fillrate_y";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQWidgetBase::SetFillRateY(double value)
{
	void *ptr[3];
	ptr[0] = "fillrate_y";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

double MQWidgetBase::GetFillBeforeRate()
{
	double value = 0;
	void *ptr[3];
	ptr[0] = "fillbeforerate";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQWidgetBase::SetFillBeforeRate(double value)
{
	void *ptr[3];
	ptr[0] = "fillbeforerate";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

double MQWidgetBase::GetFillAfterRate()
{
	double value = 0;
	void *ptr[3];
	ptr[0] = "fillafterrate";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQWidgetBase::SetFillAfterRate(double value)
{
	void *ptr[3];
	ptr[0] = "fillafterrate";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

double MQWidgetBase::GetInSpace()
{
	double value = 0;
	void *ptr[3];
	ptr[0] = "inspace";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQWidgetBase::SetInSpace(double value)
{
	void *ptr[3];
	ptr[0] = "inspace";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

double MQWidgetBase::GetOutSpace()
{
	double value = 0;
	void *ptr[3];
	ptr[0] = "outspace";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQWidgetBase::SetOutSpace(double value)
{
	void *ptr[3];
	ptr[0] = "outspace";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

double MQWidgetBase::GetHintSizeRateX()
{
	double value = 0;
	void *ptr[3];
	ptr[0] = "hintsizeratex";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQWidgetBase::SetHintSizeRateX(double value)
{
	void *ptr[3];
	ptr[0] = "hintsizeratex";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

double MQWidgetBase::GetHintSizeRateY()
{
	double value = 0;
	void *ptr[3];
	ptr[0] = "hintsizeratey";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQWidgetBase::SetHintSizeRateY(double value)
{
	void *ptr[3];
	ptr[0] = "hintsizeratey";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

POINT MQWidgetBase::GetJustSize(int max_width, int max_height)
{
	int values[4] = {0, 0, max_width, max_height};
	void *ptr[3];
	ptr[0] = "justsize";
	ptr[1] = values;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	POINT pt;
	pt.x = values[0];
	pt.y = values[1];
	return pt;
}

MQWidgetBase *MQWidgetBase::FindWidgetByID(int id)
{
	std::map<int, MQWidgetBase*>::iterator it = s_WidgetIDMap.find(id);
	if(it != s_WidgetIDMap.end()){
		return (*it).second;
	}
	return NULL;
}

int MQWidgetBase::GetSystemWidgetID(MQSystemWidget::WidgetType type)
{
	void *ptr[5];
	int result = -1;

	ptr[0] = "system";
	switch(type){
	case MQSystemWidget::MainWindow: ptr[1] = "main_window"; break;
	case MQSystemWidget::OptionPanel: ptr[1] = "option_panel"; break;
	default: return NullID;
	}
	ptr[2] = "result";
	ptr[3] = &result;
	ptr[4] = NULL;
	MQWidget_Value(NullID, MQWIDGET_FIND, (void*)ptr);
	
	return result;
}

void MQWidgetBase::GetDefaultFrameColor(int& r, int& g, int& b, int& a)
{
	int array[4] = {0,0,0,0};
	void *ptr[3];
	ptr[0] = "defaultframecolor.rgba";
	ptr[1] = array;
	ptr[2] = NULL;
	MQWidget_Value(NullID, MQWIDGET_GET, (void*)ptr);

	r = array[0];
	g = array[1];
	b = array[2];
	a = array[3];
}

void MQWidgetBase::AddEventCallback(const char *event_type, MQWidgetSharedPtr<MQWidgetEventClosureBase> closure, bool prior)
{
	void *ptr[9];

	ptr[0] = "type";
	ptr[1] = (void*)event_type;
	ptr[2] = "proc";
	ptr[3] = EventCallback;
	ptr[4] = "pointer";
	ptr[5] = closure.get();
	ptr[6] = "prior";
	ptr[7] = &prior;
	ptr[8] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_ADD_EVENT, (void*)ptr);

	m_Events.push_back(closure);
}

void MQWidgetBase::RemoveEventCallback(const char *event_type, MQWidgetSharedPtr<MQWidgetEventClosureBase> closure)
{
	std::vector<MQWidgetSharedPtr<MQWidgetEventClosureBase>>::iterator it;
	for(it = m_Events.begin(); it != m_Events.end(); ++it){
		if((*it)->isEqual(closure.get())){
			void *ptr[7];

			ptr[0] = "type";
			ptr[1] = (void*)event_type;
			ptr[2] = "proc";
			ptr[3] = EventCallback;
			ptr[4] = "pointer";
			ptr[5] = (*it).get();
			ptr[6] = NULL;
			MQWidget_Value(m_ID, MQWIDGET_REMOVE_EVENT, (void*)ptr);

			m_Events.erase(it);
			break;
		}
	}
}

bool MQWidgetBase::ExistsEventCallback(const char *event_type, MQWidgetSharedPtr<MQWidgetEventClosureBase> closure)
{
	std::vector<MQWidgetSharedPtr<MQWidgetEventClosureBase>>::iterator it;
	for(it = m_Events.begin(); it != m_Events.end(); ++it){
		if((*it)->isEqual(closure.get())){
			return true;
		}
	}
	return false;
}

BOOL MQAPICALL MQWidgetBase::EventCallback(MQDocument doc, void **params, void *option)
{
	int target_id = ExtractEventOptionInt(params, "target_id", NullID);
	int sender_id = ExtractEventOptionInt(params, "sender_id", NullID);
	const char *type = (const char *)ExtractEventOption(params, "type");
	if(type == NULL) return FALSE;

	MQWidgetBase *target = FindWidgetByID(target_id);
	if(target != NULL){
		for(size_t i=0; i<target->m_Events.size(); i++){
			if(target->m_Events[i].get() == option){
				MQWidgetBase *sender = FindWidgetByID(sender_id);
				if( strcmp(type, "leftdown") == 0 || 
					strcmp(type, "leftup") == 0 || 
					strcmp(type, "leftdoubleclick") == 0 || 
					strcmp(type, "middledown") == 0 || 
					strcmp(type, "middleup") == 0 || 
					strcmp(type, "middledoubleclick") == 0 || 
					strcmp(type, "rightdown") == 0 || 
					strcmp(type, "rightup") == 0 || 
					strcmp(type, "rightdoubleclick") == 0 || 
					strcmp(type, "mousemove") == 0 || 
					strcmp(type, "mousewheel") == 0 )
				{
					MQWidgetMouseParam param;
					param.ClientPos.x = ExtractEventOptionInt(params, "client_x", 0);
					param.ClientPos.y = ExtractEventOptionInt(params, "client_y", 0);
					param.ScreenPos.x = ExtractEventOptionInt(params, "screen_x", 0);
					param.ScreenPos.y = ExtractEventOptionInt(params, "screen_y", 0);
					param.DownPos.x = ExtractEventOptionInt(params, "down_x", 0);
					param.DownPos.y = ExtractEventOptionInt(params, "down_y", 0);
					param.LastPos.x = ExtractEventOptionInt(params, "last_x", 0);
					param.LastPos.y = ExtractEventOptionInt(params, "last_y", 0);
					param.Wheel = ExtractEventOptionInt(params, "wheel_delta", 0);

					DWORD button_state = (DWORD)ExtractEventOptionInt(params, "button_state", 0);
					param.LButton = (button_state & MK_LBUTTON) ? TRUE : FALSE;
					param.MButton = (button_state & MK_MBUTTON) ? TRUE : FALSE;
					param.RButton = (button_state & MK_RBUTTON) ? TRUE : FALSE;
					param.Shift = (button_state & MK_SHIFT) ? TRUE : FALSE;
					param.Ctrl = (button_state & MK_CONTROL) ? TRUE : FALSE;
					param.Alt = (button_state & MK_ALT) ? TRUE : FALSE;

					return target->m_Events[i]->invoke(sender, doc, &param);
				}
				else if(
					strcmp(type, "keydown") == 0 || 
					strcmp(type, "keyup") == 0 )
				{
					MQWidgetKeyParam param;
					param.Key = ExtractEventOptionInt(params, "key", 0);
					param.AutoRepeat = ExtractEventOptionInt(params, "auto_repeat", 0) != 0;

					DWORD button_state = (DWORD)ExtractEventOptionInt(params, "button_state", 0);
					param.LButton = (button_state & MK_LBUTTON) ? TRUE : FALSE;
					param.MButton = (button_state & MK_MBUTTON) ? TRUE : FALSE;
					param.RButton = (button_state & MK_RBUTTON) ? TRUE : FALSE;
					param.Shift = (button_state & MK_SHIFT) ? TRUE : FALSE;
					param.Ctrl = (button_state & MK_CONTROL) ? TRUE : FALSE;
					param.Alt = (button_state & MK_ALT) ? TRUE : FALSE;

					return target->m_Events[i]->invoke(sender, doc, &param);
				}
				else
				{
					return target->m_Events[i]->invoke(sender, doc, NULL);
				}
			}
		}
	}

	return FALSE;
}

//---------------------------------------------------------------------------
//  class MQFrameBase
//---------------------------------------------------------------------------
MQFrameBase::MQFrameBase() : MQWidgetBase()
{
}

MQFrameBase::MQFrameBase(int id) : MQWidgetBase(id)
{
}

MQFrameBase::~MQFrameBase()
{
}

MQFrameBase::MQFrameAlignment MQFrameBase::GetAlignment()
{
	char *align_str = NULL;
	void *ptr[3];
	ptr[0] = "alignment";
	ptr[1] = &align_str;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	if(strcmp(align_str, "horz") == 0) return ALIGN_HORIZONTAL;
	if(strcmp(align_str, "vert") == 0) return ALIGN_VERTICAL;
	return ALIGN_NONE;
}

void MQFrameBase::SetAlignment(MQFrameAlignment align)
{
	void *ptr[3];
	ptr[0] = "alignment";
	switch(align){
	case ALIGN_HORIZONTAL: ptr[1] = "horz"; break;
	case ALIGN_VERTICAL:   ptr[1] = "vert"; break;
	default: ptr[1] = "none"; break;
	}
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

bool MQFrameBase::GetMultiColumn()
{
	bool value = false;
	void *ptr[3];
	ptr[0] = "multicolumn";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQFrameBase::SetMultiColumn(bool value)
{
	void *ptr[3];
	ptr[0] = "multicolumn";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

int MQFrameBase::GetMatrixColumn()
{
	int value = 0;
	void *ptr[3];
	ptr[0] = "matrixcolumn";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQFrameBase::SetMatrixColumn(int value)
{
	void *ptr[3];
	ptr[0] = "matrixcolumn";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

bool MQFrameBase::GetUniformSize()
{
	bool value = false;
	void *ptr[3];
	ptr[0] = "uniformsize";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQFrameBase::SetUniformSize(bool value)
{
	void *ptr[3];
	ptr[0] = "uniformsize";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

bool MQFrameBase::GetSplit()
{
	bool value = false;
	void *ptr[3];
	ptr[0] = "split";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQFrameBase::SetSplit(bool value)
{
	void *ptr[3];
	ptr[0] = "split";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}


//---------------------------------------------------------------------------
//  class MQFrame
//---------------------------------------------------------------------------
MQFrame::MQFrame() : MQFrameBase()
{
	void *ptr[5];
	ptr[0] = "type";
	ptr[1] = "frame";
	ptr[2] = "id";
	ptr[3] = &m_ID;
	ptr[4] = NULL;
	MQWidget_Value(NullID, MQWIDGET_CREATE, (void*)ptr);

	if(m_ID != NullID){
		m_IDOwner = true;
		s_WidgetIDMap.insert(std::pair<int,MQWidgetBase*>(m_ID, this));
	}
}

MQFrame::MQFrame(int id) : MQFrameBase(id)
{
}

MQFrame::~MQFrame()
{
}

void MQFrame::GetBackColor(int& r, int& g, int& b, int& a)
{
	int array[4] = {0,0,0,0};
	void *ptr[3];
	ptr[0] = "backcolor.rgba";
	ptr[1] = array;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	r = array[0];
	g = array[1];
	b = array[2];
	a = array[3];
}

void MQFrame::SetBackColor(int r, int g, int b, int a)
{
	int array[4] = {r,g,b,a};
	void *ptr[9];
	ptr[0] = "backcolor.rgba";
	ptr[1] = array;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}


//---------------------------------------------------------------------------
//  class MQGroupBox
//---------------------------------------------------------------------------
MQGroupBox::MQGroupBox() : MQFrameBase()
{
	void *ptr[5];
	ptr[0] = "type";
	ptr[1] = "groupbox";
	ptr[2] = "id";
	ptr[3] = &m_ID;
	ptr[4] = NULL;
	MQWidget_Value(NullID, MQWIDGET_CREATE, (void*)ptr);

	if(m_ID != NullID){
		m_IDOwner = true;
		s_WidgetIDMap.insert(std::pair<int,MQWidgetBase*>(m_ID, this));
	}
}

MQGroupBox::MQGroupBox(int id) : MQFrameBase(id)
{
}

MQGroupBox::~MQGroupBox()
{
}

std::wstring MQGroupBox::GetText()
{
	void *ptr[3];
	unsigned int length = 0;

	ptr[0] = "text.length";
	ptr[1] = &length;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	if(length == 0){
		return std::wstring();
	}

	wchar_t *buf = new wchar_t[length+1];
	ptr[0] = "text";
	ptr[1] = buf;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	std::wstring ret(buf);
	delete[] buf;
	return ret;
}

void MQGroupBox::SetText(const std::wstring& text)
{
	void *ptr[3];
	ptr[0] = "text";
	ptr[1] = (void*)text.c_str();
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

bool MQGroupBox::GetCanMinimize()
{
	bool value = false;
	void *ptr[3];
	ptr[0] = "canminimize";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQGroupBox::SetCanMinimize(bool value)
{
	void *ptr[3];
	ptr[0] = "canminimize";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

bool MQGroupBox::GetMinimized()
{
	bool value = false;
	void *ptr[3];
	ptr[0] = "minimized";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQGroupBox::SetMinimized(bool value)
{
	void *ptr[3];
	ptr[0] = "minimized";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

bool MQGroupBox::GetShowTitle()
{
	bool value = false;
	void *ptr[3];
	ptr[0] = "showtitle";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQGroupBox::SetShowTitle(bool value)
{
	void *ptr[3];
	ptr[0] = "showtitle";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}


//---------------------------------------------------------------------------
//  class MQScrollBox
//---------------------------------------------------------------------------
MQScrollBox::MQScrollBox() : MQWidgetBase()
{
	void *ptr[5];
	ptr[0] = "type";
	ptr[1] = "scrollbox";
	ptr[2] = "id";
	ptr[3] = &m_ID;
	ptr[4] = NULL;
	MQWidget_Value(NullID, MQWIDGET_CREATE, (void*)ptr);

	if(m_ID != NullID){
		m_IDOwner = true;
		s_WidgetIDMap.insert(std::pair<int,MQWidgetBase*>(m_ID, this));
	}
}

MQScrollBox::MQScrollBox(int id) : MQWidgetBase(id)
{
}

MQScrollBox::~MQScrollBox()
{
}

MQScrollBox::MQScrollBoxBarStatus MQScrollBox::GetHorzBarStatus()
{
	char *status_str = NULL;
	void *ptr[3];
	ptr[0] = "horzbarstatus";
	ptr[1] = &status_str;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	if(strcmp(status_str, "auto") == 0) return MQScrollBox::SCROLLBAR_AUTO;
	if(strcmp(status_str, "on") == 0) return MQScrollBox::SCROLLBAR_ON;
	if(strcmp(status_str, "off") == 0) return MQScrollBox::SCROLLBAR_OFF;
	return MQScrollBox::SCROLLBAR_AUTO;
}

void MQScrollBox::SetHorzBarStatus(MQScrollBoxBarStatus value)
{
	void *ptr[3];
	ptr[0] = "horzbarstatus";
	switch(value){
	default:
	case SCROLLBAR_AUTO: ptr[1] = "auto"; break;
	case SCROLLBAR_OFF:  ptr[1] = "off"; break;
	case SCROLLBAR_ON:   ptr[1] = "on"; break;
	}
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

MQScrollBox::MQScrollBoxBarStatus MQScrollBox::GetVertBarStatus()
{
	char *status_str = NULL;
	int value = false;
	void *ptr[3];
	ptr[0] = "vertbarstatus";
	ptr[1] = &status_str;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	if(strcmp(status_str, "auto") == 0) return MQScrollBox::SCROLLBAR_AUTO;
	if(strcmp(status_str, "on") == 0) return MQScrollBox::SCROLLBAR_ON;
	if(strcmp(status_str, "off") == 0) return MQScrollBox::SCROLLBAR_OFF;
	return MQScrollBox::SCROLLBAR_AUTO;
}

void MQScrollBox::SetVertBarStatus(MQScrollBoxBarStatus value)
{
	void *ptr[3];
	ptr[0] = "vertbarstatus";
	switch(value){
	default:
	case SCROLLBAR_AUTO: ptr[1] = "auto"; break;
	case SCROLLBAR_OFF:  ptr[1] = "off"; break;
	case SCROLLBAR_ON:   ptr[1] = "on"; break;
	}
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

bool MQScrollBox::GetAutoWidgetScroll()
{
	bool value = false;
	void *ptr[3];
	ptr[0] = "autowidgetscroll";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQScrollBox::SetAutoWidgetScroll(bool value)
{
	void *ptr[3];
	ptr[0] = "autowidgetscroll";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}



//---------------------------------------------------------------------------
//  class MQTab
//---------------------------------------------------------------------------
MQTab::MQTab() : MQWidgetBase()
{
	void *ptr[5];
	ptr[0] = "type";
	ptr[1] = "tab";
	ptr[2] = "id";
	ptr[3] = &m_ID;
	ptr[4] = NULL;
	MQWidget_Value(NullID, MQWIDGET_CREATE, (void*)ptr);

	if(m_ID != NullID){
		m_IDOwner = true;
		s_WidgetIDMap.insert(std::pair<int,MQWidgetBase*>(m_ID, this));
	}
}

MQTab::MQTab(int id) : MQWidgetBase(id)
{
}

MQTab::~MQTab()
{
}

std::wstring MQTab::GetTabTitle(int index)
{
	void *ptr[5];
	unsigned int length = 0;

	ptr[0] = "index";
	ptr[1] = &index;
	ptr[2] = "title.text.length";
	ptr[3] = &length;
	ptr[4] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	if(length == 0){
		return std::wstring();
	}

	wchar_t *buf = new wchar_t[length+1];
	ptr[2] = "title.text";
	ptr[3] = buf;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	std::wstring ret(buf);
	delete[] buf;
	return ret;
}

void MQTab::SetTabTitle(int index, const std::wstring& text)
{
	void *ptr[5];
	ptr[0] = "index";
	ptr[1] = &index;
	ptr[2] = "title.text";
	ptr[3] = (void*)text.c_str();
	ptr[4] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

int MQTab::GetCurrentPage()
{
	int value = 0;
	void *ptr[3];
	ptr[0] = "currentpage";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQTab::SetCurrentPage(bool value)
{
	void *ptr[3];
	ptr[0] = "currentpage";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

bool MQTab::GetShowTab()
{
	bool value = false;
	void *ptr[3];
	ptr[0] = "showtab";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQTab::SetShowTab(bool value)
{
	void *ptr[3];
	ptr[0] = "showtab";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

bool MQTab::GetExclusive()
{
	bool value = false;
	void *ptr[3];
	ptr[0] = "exclusive";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQTab::SetExclusive(bool value)
{
	void *ptr[3];
	ptr[0] = "exclusive";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}


//---------------------------------------------------------------------------
//  class MQButton
//---------------------------------------------------------------------------
MQButton::MQButton() : MQWidgetBase()
{
	void *ptr[5];
	ptr[0] = "type";
	ptr[1] = "button";
	ptr[2] = "id";
	ptr[3] = &m_ID;
	ptr[4] = NULL;
	MQWidget_Value(NullID, MQWIDGET_CREATE, (void*)ptr);

	if(m_ID != NullID){
		m_IDOwner = true;
		s_WidgetIDMap.insert(std::pair<int,MQWidgetBase*>(m_ID, this));
	}
}

MQButton::MQButton(int id) : MQWidgetBase(id)
{
}

MQButton::~MQButton()
{
}

std::wstring MQButton::GetText()
{
	void *ptr[3];
	unsigned int length = 0;

	ptr[0] = "text.length";
	ptr[1] = &length;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	if(length == 0){
		return std::wstring();
	}

	wchar_t *buf = new wchar_t[length+1];
	ptr[0] = "text";
	ptr[1] = buf;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	std::wstring ret(buf);
	delete[] buf;
	return ret;
}

void MQButton::SetText(const std::wstring& text)
{
	void *ptr[3];
	ptr[0] = "text";
	ptr[1] = (void*)text.c_str();
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

bool MQButton::GetToggle()
{
	bool value = false;
	void *ptr[3];
	ptr[0] = "toggle";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQButton::SetToggle(bool value)
{
	void *ptr[3];
	ptr[0] = "toggle";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

bool MQButton::GetDown()
{
	bool value = false;
	void *ptr[3];
	ptr[0] = "down";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQButton::SetDown(bool value)
{
	void *ptr[3];
	ptr[0] = "down";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

bool MQButton::GetRepeat()
{
	bool value = false;
	void *ptr[3];
	ptr[0] = "repeat";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQButton::SetRepeat(bool value)
{
	void *ptr[3];
	ptr[0] = "repeat";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

bool MQButton::GetChain()
{
	bool value = false;
	void *ptr[3];
	ptr[0] = "chain";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQButton::SetChain(bool value)
{
	void *ptr[3];
	ptr[0] = "chain";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

void MQButton::SetDefault(bool value)
{
	void *ptr[3];
	ptr[0] = "default";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

void MQButton::SetCancel(bool value)
{
	void *ptr[3];
	ptr[0] = "cancel";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

void MQButton::SetModalResult(MQDialog::DIALOG_RESULT value)
{
	void *ptr[3];
	ptr[0] = "modalresult";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}


//---------------------------------------------------------------------------
//  class MQCheckBox
//---------------------------------------------------------------------------
MQCheckBox::MQCheckBox() : MQWidgetBase()
{
	void *ptr[5];
	ptr[0] = "type";
	ptr[1] = "checkbox";
	ptr[2] = "id";
	ptr[3] = &m_ID;
	ptr[4] = NULL;
	MQWidget_Value(NullID, MQWIDGET_CREATE, (void*)ptr);

	if(m_ID != NullID){
		m_IDOwner = true;
		s_WidgetIDMap.insert(std::pair<int,MQWidgetBase*>(m_ID, this));
	}
}

MQCheckBox::MQCheckBox(int id) : MQWidgetBase(id)
{
}

MQCheckBox::~MQCheckBox()
{
}

bool MQCheckBox::GetChecked()
{
	bool value = false;
	void *ptr[3];
	ptr[0] = "checked";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQCheckBox::SetChecked(bool value)
{
	void *ptr[3];
	ptr[0] = "checked";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

std::wstring MQCheckBox::GetText()
{
	void *ptr[3];
	unsigned int length = 0;

	ptr[0] = "text.length";
	ptr[1] = &length;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	if(length == 0){
		return std::wstring();
	}

	wchar_t *buf = new wchar_t[length+1];
	ptr[0] = "text";
	ptr[1] = buf;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	std::wstring ret(buf);
	delete[] buf;
	return ret;
}

void MQCheckBox::SetText(const std::wstring& text)
{
	void *ptr[3];
	ptr[0] = "text";
	ptr[1] = (void*)text.c_str();
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}


//---------------------------------------------------------------------------
//  class MQRadioButton
//---------------------------------------------------------------------------
MQRadioButton::MQRadioButton() : MQWidgetBase()
{
	void *ptr[5];
	ptr[0] = "type";
	ptr[1] = "radiobutton";
	ptr[2] = "id";
	ptr[3] = &m_ID;
	ptr[4] = NULL;
	MQWidget_Value(NullID, MQWIDGET_CREATE, (void*)ptr);

	if(m_ID != NullID){
		m_IDOwner = true;
		s_WidgetIDMap.insert(std::pair<int,MQWidgetBase*>(m_ID, this));
	}
}

MQRadioButton::MQRadioButton(int id) : MQWidgetBase(id)
{
}

MQRadioButton::~MQRadioButton()
{
}

bool MQRadioButton::GetChecked()
{
	bool value = false;
	void *ptr[3];
	ptr[0] = "checked";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQRadioButton::SetChecked(bool value)
{
	void *ptr[3];
	ptr[0] = "checked";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

std::wstring MQRadioButton::GetText()
{
	void *ptr[3];
	unsigned int length = 0;

	ptr[0] = "text.length";
	ptr[1] = &length;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	if(length == 0){
		return std::wstring();
	}

	wchar_t *buf = new wchar_t[length+1];
	ptr[0] = "text";
	ptr[1] = buf;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	std::wstring ret(buf);
	delete[] buf;
	return ret;
}

void MQRadioButton::SetText(const std::wstring& text)
{
	void *ptr[3];
	ptr[0] = "text";
	ptr[1] = (void*)text.c_str();
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}


//---------------------------------------------------------------------------
//  class MQComboBox
//---------------------------------------------------------------------------
MQComboBox::MQComboBox() : MQWidgetBase()
{
	void *ptr[5];
	ptr[0] = "type";
	ptr[1] = "combobox";
	ptr[2] = "id";
	ptr[3] = &m_ID;
	ptr[4] = NULL;
	MQWidget_Value(NullID, MQWIDGET_CREATE, (void*)ptr);

	if(m_ID != NullID){
		m_IDOwner = true;
		s_WidgetIDMap.insert(std::pair<int,MQWidgetBase*>(m_ID, this));
	}
}

MQComboBox::MQComboBox(int id) : MQWidgetBase(id)
{
}

MQComboBox::~MQComboBox()
{
}

int MQComboBox::GetCurrentIndex()
{
	int value = 0;
	void *ptr[3];
	ptr[0] = "currentindex";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQComboBox::SetCurrentIndex(int value)
{
	void *ptr[3];
	ptr[0] = "currentindex";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

int MQComboBox::AddItem(const std::wstring& text)
{
	int result = -1;
	void *ptr[7];
	ptr[0] = "additem";
	ptr[1] = NULL;
	ptr[2] = "text";
	ptr[3] = (void*)text.c_str();
	ptr[4] = "result";
	ptr[5] = &result;
	ptr[6] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_EDIT, (void*)ptr);
	return result;
}

int MQComboBox::AddItem(const std::wstring& text, __int64 tag)
{
	int result = -1;
	void *ptr[9];
	ptr[0] = "additem";
	ptr[1] = NULL;
	ptr[2] = "text";
	ptr[3] = (void*)text.c_str();
	ptr[4] = "result";
	ptr[5] = &result;
	ptr[6] = "tag";
	ptr[7] = &tag;
	ptr[8] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_EDIT, (void*)ptr);
	return result;
}

void MQComboBox::DeleteItem(int index)
{
	void *ptr[5];
	ptr[0] = "deleteitem";
	ptr[1] = NULL;
	ptr[2] = "index";
	ptr[3] = &index;
	ptr[4] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_EDIT, (void*)ptr);
}

void MQComboBox::ClearItems()
{
	void *ptr[3];
	ptr[0] = "clearitems";
	ptr[1] = NULL;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_EDIT, (void*)ptr);
}

int MQComboBox::GetItemCount()
{
	int value = 0;
	void *ptr[3];

	ptr[0] = "item.count";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);
	return value;
}

std::wstring MQComboBox::GetItem(int index)
{
	void *ptr[5];
	unsigned int length = 0;

	ptr[0] = "index";
	ptr[1] = &index;
	ptr[2] = "item.text.length";
	ptr[3] = &length;
	ptr[4] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	if(length == 0){
		return std::wstring();
	}

	wchar_t *buf = new wchar_t[length+1];
	ptr[2] = "item.text";
	ptr[3] = buf;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	std::wstring ret(buf);
	delete[] buf;
	return ret;
}

void MQComboBox::SetItem(int index, const std::wstring& text)
{
	void *ptr[5];
	ptr[0] = "index";
	ptr[1] = &index;
	ptr[2] = "item.text";
	ptr[3] = (void*)text.c_str();
	ptr[4] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

__int64 MQComboBox::GetItemTag(int index)
{
	__int64 value = 0;
	void *ptr[5];
	ptr[0] = "index";
	ptr[1] = &index;
	ptr[2] = "item.tag";
	ptr[3] = &value;
	ptr[4] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQComboBox::SetItemTag(int index, __int64 tag)
{
	void *ptr[5];
	ptr[0] = "index";
	ptr[1] = &index;
	ptr[2] = "item.tag";
	ptr[3] = &tag;
	ptr[4] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

int MQComboBox::GetNumVisible()
{
	int  value = 0;
	void *ptr[3];
	ptr[0] = "numvisible";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQComboBox::SetNumVisible(int value)
{
	void *ptr[3];
	ptr[0] = "numvisible";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}


//---------------------------------------------------------------------------
//  class MQListBox
//---------------------------------------------------------------------------
MQListBox::MQListBox() : MQWidgetBase()
{
	void *ptr[5];
	ptr[0] = "type";
	ptr[1] = "listbox";
	ptr[2] = "id";
	ptr[3] = &m_ID;
	ptr[4] = NULL;
	MQWidget_Value(NullID, MQWIDGET_CREATE, (void*)ptr);

	if(m_ID != NullID){
		m_IDOwner = true;
		s_WidgetIDMap.insert(std::pair<int,MQWidgetBase*>(m_ID, this));
	}
}

MQListBox::MQListBox(int id) : MQWidgetBase(id)
{
}

MQListBox::~MQListBox()
{
}

int MQListBox::GetCurrentIndex()
{
	int value = 0;
	void *ptr[3];
	ptr[0] = "currentindex";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQListBox::SetCurrentIndex(int value)
{
	void *ptr[3];
	ptr[0] = "currentindex";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

int MQListBox::AddItem(const std::wstring& text)
{
	int result = -1;
	void *ptr[7];
	ptr[0] = "additem";
	ptr[1] = NULL;
	ptr[2] = "text";
	ptr[3] = (void*)text.c_str();
	ptr[4] = "result";
	ptr[5] = &result;
	ptr[6] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_EDIT, (void*)ptr);
	return result;
}

int MQListBox::AddItem(const std::wstring& text, __int64 tag)
{
	int result = -1;
	void *ptr[9];
	ptr[0] = "additem";
	ptr[1] = NULL;
	ptr[2] = "text";
	ptr[3] = (void*)text.c_str();
	ptr[4] = "result";
	ptr[5] = &result;
	ptr[6] = "tag";
	ptr[7] = &tag;
	ptr[8] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_EDIT, (void*)ptr);
	return result;
}

void MQListBox::DeleteItem(int index)
{
	void *ptr[5];
	ptr[0] = "deleteitem";
	ptr[1] = NULL;
	ptr[2] = "index";
	ptr[3] = &index;
	ptr[4] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_EDIT, (void*)ptr);
}

void MQListBox::ClearItems()
{
	void *ptr[3];
	ptr[0] = "clearitems";
	ptr[1] = NULL;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_EDIT, (void*)ptr);
}

int MQListBox::GetItemCount()
{
	int value = 0;
	void *ptr[3];

	ptr[0] = "item.count";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);
	return value;
}

std::wstring MQListBox::GetItem(int index)
{
	void *ptr[5];
	unsigned int length = 0;

	ptr[0] = "index";
	ptr[1] = &index;
	ptr[2] = "item.text.length";
	ptr[3] = &length;
	ptr[4] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	if(length == 0){
		return std::wstring();
	}

	wchar_t *buf = new wchar_t[length+1];
	ptr[2] = "item.text";
	ptr[3] = buf;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	std::wstring ret(buf);
	delete[] buf;
	return ret;
}

void MQListBox::SetItem(int index, const std::wstring& text)
{
	void *ptr[5];
	ptr[0] = "index";
	ptr[1] = &index;
	ptr[2] = "item.text";
	ptr[3] = (void*)text.c_str();
	ptr[4] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

__int64 MQListBox::GetItemTag(int index)
{
	__int64 value = 0;
	void *ptr[5];
	ptr[0] = "index";
	ptr[1] = &index;
	ptr[2] = "item.tag";
	ptr[3] = &value;
	ptr[4] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQListBox::SetItemTag(int index, __int64 tag)
{
	void *ptr[5];
	ptr[0] = "index";
	ptr[1] = &index;
	ptr[2] = "item.tag";
	ptr[3] = &tag;
	ptr[4] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

bool MQListBox::GetItemSelected(int index)
{
	bool value = false;
	void *ptr[5];
	ptr[0] = "index";
	ptr[1] = &index;
	ptr[2] = "item.selected";
	ptr[3] = &value;
	ptr[4] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQListBox::SetItemSelected(int index, bool selected)
{
	void *ptr[5];
	ptr[0] = "index";
	ptr[1] = &index;
	ptr[2] = "item.selected";
	ptr[3] = &selected;
	ptr[4] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

bool MQListBox::GetMultiSelect()
{
	bool value = false;
	void *ptr[3];
	ptr[0] = "multiselect";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQListBox::SetMultiSelect(bool value)
{
	void *ptr[3];
	ptr[0] = "multiselect";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

void MQListBox::ClearSelection()
{
	void *ptr[3];
	ptr[0] = "clearselection";
	ptr[1] = NULL;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_EDIT, (void*)ptr);
}

void MQListBox::MakeItemVisible(int index)
{
	void *ptr[5];
	ptr[0] = "makeitemvisible";
	ptr[1] = NULL;
	ptr[2] = "index";
	ptr[3] = &index;
	ptr[4] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_EDIT, (void*)ptr);
}


//---------------------------------------------------------------------------
//  class MQLabel
//---------------------------------------------------------------------------
MQLabel::MQLabel() : MQWidgetBase()
{
	void *ptr[5];
	ptr[0] = "type";
	ptr[1] = "label";
	ptr[2] = "id";
	ptr[3] = &m_ID;
	ptr[4] = NULL;
	MQWidget_Value(NullID, MQWIDGET_CREATE, (void*)ptr);

	if(m_ID != NullID){
		m_IDOwner = true;
		s_WidgetIDMap.insert(std::pair<int,MQWidgetBase*>(m_ID, this));
	}
}

MQLabel::MQLabel(int id) : MQWidgetBase(id)
{
}

MQLabel::~MQLabel()
{
}

std::wstring MQLabel::GetText()
{
	void *ptr[3];
	unsigned int length = 0;

	ptr[0] = "text.length";
	ptr[1] = &length;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	if(length == 0){
		return std::wstring();
	}

	wchar_t *buf = new wchar_t[length+1];
	ptr[0] = "text";
	ptr[1] = buf;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	std::wstring ret(buf);
	delete[] buf;
	return ret;
}

void MQLabel::SetText(const std::wstring& text)
{
	void *ptr[3];
	ptr[0] = "text";
	ptr[1] = (void*)text.c_str();
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

MQLabel::MQLabelTextAlignment MQLabel::GetAlignment()
{
	char *align_str = NULL;
	void *ptr[3];
	ptr[0] = "alignment";
	ptr[1] = &align_str;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	if(strcmp(align_str, "left") == 0) return ALIGN_LEFT;
	if(strcmp(align_str, "center") == 0) return ALIGN_CENTER;
	if(strcmp(align_str, "right") == 0) return ALIGN_RIGHT;
	return ALIGN_LEFT;
}

void MQLabel::SetAlignment(MQLabelTextAlignment value)
{
	void *ptr[3];
	ptr[0] = "alignment";
	switch(value){
	default:
	case ALIGN_LEFT:   ptr[1] = "left"; break;
	case ALIGN_CENTER: ptr[1] = "center"; break;
	case ALIGN_RIGHT:  ptr[1] = "right"; break;
	}
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

bool MQLabel::GetVerticalText()
{
	bool value = false;
	void *ptr[3];
	ptr[0] = "verticaltext";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQLabel::SetVerticalText(bool value)
{
	void *ptr[3];
	ptr[0] = "verticaltext";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

bool MQLabel::GetWordWrap()
{
	bool value = false;
	void *ptr[3];
	ptr[0] = "wordwrap";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQLabel::SetWordWrap(bool value)
{
	void *ptr[3];
	ptr[0] = "wordwrap";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

bool MQLabel::GetFrame()
{
	bool value = false;
	void *ptr[3];
	ptr[0] = "frame";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQLabel::SetFrame(bool value)
{
	void *ptr[3];
	ptr[0] = "frame";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}


//---------------------------------------------------------------------------
//  class MQEdit
//---------------------------------------------------------------------------
MQEdit::MQEdit() : MQWidgetBase()
{
	void *ptr[5];
	ptr[0] = "type";
	ptr[1] = "edit";
	ptr[2] = "id";
	ptr[3] = &m_ID;
	ptr[4] = NULL;
	MQWidget_Value(NullID, MQWIDGET_CREATE, (void*)ptr);

	if(m_ID != NullID){
		m_IDOwner = true;
		s_WidgetIDMap.insert(std::pair<int,MQWidgetBase*>(m_ID, this));
	}
}

MQEdit::MQEdit(int id) : MQWidgetBase(id)
{
}

MQEdit::~MQEdit()
{
}

std::wstring MQEdit::GetText()
{
	void *ptr[3];
	unsigned int length = 0;

	ptr[0] = "text.length";
	ptr[1] = &length;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	if(length == 0){
		return std::wstring();
	}

	wchar_t *buf = new wchar_t[length+1];
	ptr[0] = "text";
	ptr[1] = buf;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	std::wstring ret(buf);
	delete[] buf;
	return ret;
}

void MQEdit::SetText(const std::wstring& text)
{
	void *ptr[3];
	ptr[0] = "text";
	ptr[1] = (void*)text.c_str();
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

bool MQEdit::GetReadOnly()
{
	bool value = false;
	void *ptr[3];
	ptr[0] = "readonly";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQEdit::SetReadOnly(bool value)
{
	void *ptr[3];
	ptr[0] = "readonly";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

bool MQEdit::GetPassword()
{
	bool value = false;
	void *ptr[3];
	ptr[0] = "password";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQEdit::SetPassword(bool value)
{
	void *ptr[3];
	ptr[0] = "password";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

MQEdit::MQEditTextAlignment MQEdit::GetAlignment()
{
	char *align_str = NULL;
	void *ptr[3];
	ptr[0] = "alignment";
	ptr[1] = &align_str;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	if(strcmp(align_str, "left") == 0) return ALIGN_LEFT;
	if(strcmp(align_str, "right") == 0) return ALIGN_RIGHT;
	return ALIGN_LEFT;
}

void MQEdit::SetAlignment(MQEditTextAlignment value)
{
	void *ptr[3];
	ptr[0] = "alignment";
	switch(value){
	default:
	case ALIGN_LEFT:   ptr[1] = "left"; break;
	case ALIGN_RIGHT:  ptr[1] = "right"; break;
	}
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

MQEdit::MQEditNumericType MQEdit::GetNumeric()
{
	char *align_str = NULL;
	void *ptr[3];
	ptr[0] = "numeric";
	ptr[1] = &align_str;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	if(strcmp(align_str, "int") == 0) return NUMERIC_INT;
	if(strcmp(align_str, "double") == 0) return NUMERIC_DOUBLE;
	return NUMERIC_TEXT;
}

void MQEdit::SetNumeric(MQEditNumericType value)
{
	void *ptr[3];
	ptr[0] = "numeric";
	switch(value){
	default:
	case NUMERIC_TEXT:  ptr[1] = "text"; break;
	case NUMERIC_INT:   ptr[1] = "int"; break;
	case NUMERIC_DOUBLE:ptr[1] = "double"; break;
	}
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

int MQEdit::GetMaxLength()
{
	int value = 0;
	void *ptr[3];
	ptr[0] = "maxlength";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQEdit::SetMaxLength(int value)
{
	void *ptr[3];
	ptr[0] = "maxlength";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

int MQEdit::GetMaxAnsiLength()
{
	int value = 0;
	void *ptr[3];
	ptr[0] = "maxansilength";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQEdit::SetMaxAnsiLength(int value)
{
	void *ptr[3];
	ptr[0] = "maxansilength";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

int MQEdit::GetVisibleColumn()
{
	int value = 0;
	void *ptr[3];
	ptr[0] = "visiblecolumn";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQEdit::SetVisibleColumn(int value)
{
	void *ptr[3];
	ptr[0] = "visiblecolumn";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

void MQEdit::SetFocus(bool value)
{
	void *ptr[3];
	ptr[0] = "focus";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}


//---------------------------------------------------------------------------
//  class MQMemo
//---------------------------------------------------------------------------
MQMemo::MQMemo() : MQWidgetBase()
{
	void *ptr[5];
	ptr[0] = "type";
	ptr[1] = "memo";
	ptr[2] = "id";
	ptr[3] = &m_ID;
	ptr[4] = NULL;
	MQWidget_Value(NullID, MQWIDGET_CREATE, (void*)ptr);

	if(m_ID != NullID){
		m_IDOwner = true;
		s_WidgetIDMap.insert(std::pair<int,MQWidgetBase*>(m_ID, this));
	}
}

MQMemo::MQMemo(int id) : MQWidgetBase(id)
{
}

MQMemo::~MQMemo()
{
}

void MQMemo::AddLine(const std::wstring& text)
{
	void *ptr[5];
	ptr[0] = "addline";
	ptr[1] = NULL;
	ptr[2] = "text";
	ptr[3] = (void*)text.c_str();
	ptr[4] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_EDIT, (void*)ptr);
}

std::wstring MQMemo::GetLine(int line_index)
{
	wchar_t *result = NULL;
	void *ptr[7];
	unsigned int length = 0;
	ptr[0] = "getline.length";
	ptr[1] = NULL;
	ptr[2] = "line";
	ptr[3] = &line_index;
	ptr[4] = "result";
	ptr[5] = &length;
	ptr[6] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_EDIT, (void*)ptr);

	wchar_t *buf = new wchar_t[length+1];
	ptr[0] = "getline";
	ptr[1] = NULL;
	ptr[2] = "line";
	ptr[3] = &line_index;
	ptr[4] = "text";
	ptr[5] = buf;
	ptr[6] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_EDIT, (void*)ptr);

	std::wstring ret(buf);
	delete[] buf;
	return ret;
}

std::wstring MQMemo::GetText()
{
	void *ptr[3];
	unsigned int length = 0;

	ptr[0] = "text.length";
	ptr[1] = &length;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	if(length == 0){
		return std::wstring();
	}

	wchar_t *buf = new wchar_t[length+1];
	ptr[0] = "text";
	ptr[1] = buf;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	std::wstring ret(buf);
	delete[] buf;
	return ret;
}

void MQMemo::SetText(const std::wstring& text)
{
	void *ptr[3];
	ptr[0] = "text";
	ptr[1] = (void*)text.c_str();
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

bool MQMemo::GetReadOnly()
{
	bool value = false;
	void *ptr[3];
	ptr[0] = "readonly";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQMemo::SetReadOnly(bool value)
{
	void *ptr[3];
	ptr[0] = "readonly";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

void MQMemo::SetFocus(bool value)
{
	void *ptr[3];
	ptr[0] = "focus";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}


//---------------------------------------------------------------------------
//  class MQSpinBox
//---------------------------------------------------------------------------
MQSpinBox::MQSpinBox() : MQWidgetBase()
{
	void *ptr[5];
	ptr[0] = "type";
	ptr[1] = "spinbox";
	ptr[2] = "id";
	ptr[3] = &m_ID;
	ptr[4] = NULL;
	MQWidget_Value(NullID, MQWIDGET_CREATE, (void*)ptr);

	if(m_ID != NullID){
		m_IDOwner = true;
		s_WidgetIDMap.insert(std::pair<int,MQWidgetBase*>(m_ID, this));
	}
}

MQSpinBox::MQSpinBox(int id) : MQWidgetBase(id)
{
}

MQSpinBox::~MQSpinBox()
{
}

int MQSpinBox::GetPosition()
{
	int value = 0;
	void *ptr[3];
	ptr[0] = "position";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQSpinBox::SetPosition(int value)
{
	void *ptr[3];
	ptr[0] = "position";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

int MQSpinBox::GetMin()
{
	int value = 0;
	void *ptr[3];
	ptr[0] = "min";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQSpinBox::SetMin(int value)
{
	void *ptr[3];
	ptr[0] = "min";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

int MQSpinBox::GetMax()
{
	int value = 0;
	void *ptr[3];
	ptr[0] = "max";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQSpinBox::SetMax(int value)
{
	void *ptr[3];
	ptr[0] = "max";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

int MQSpinBox::GetIncrement()
{
	int value = 0;
	void *ptr[3];
	ptr[0] = "increment";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQSpinBox::SetIncrement(int value)
{
	void *ptr[3];
	ptr[0] = "increment";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

MQSpinBox::MQSpinBoxTextAlignment MQSpinBox::GetAlignment()
{
	char *align_str = NULL;
	void *ptr[3];
	ptr[0] = "alignment";
	ptr[1] = &align_str;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	if(strcmp(align_str, "left") == 0) return ALIGN_LEFT;
	if(strcmp(align_str, "right") == 0) return ALIGN_RIGHT;
	return ALIGN_LEFT;
}

void MQSpinBox::SetAlignment(MQSpinBoxTextAlignment value)
{
	void *ptr[3];
	ptr[0] = "alignment";
	switch(value){
	default:
	case ALIGN_LEFT:   ptr[1] = "left"; break;
	case ALIGN_RIGHT:  ptr[1] = "right"; break;
	}
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

int MQSpinBox::GetVisibleColumn()
{
	int value = 0;
	void *ptr[3];
	ptr[0] = "visiblecolumn";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQSpinBox::SetVisibleColumn(int value)
{
	void *ptr[3];
	ptr[0] = "visiblecolumn";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

void MQSpinBox::SetFocus(bool value)
{
	void *ptr[3];
	ptr[0] = "focus";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}


//---------------------------------------------------------------------------
//  class MQDoubleSpinBox
//---------------------------------------------------------------------------
MQDoubleSpinBox::MQDoubleSpinBox() : MQWidgetBase()
{
	void *ptr[5];
	ptr[0] = "type";
	ptr[1] = "doublespinbox";
	ptr[2] = "id";
	ptr[3] = &m_ID;
	ptr[4] = NULL;
	MQWidget_Value(NullID, MQWIDGET_CREATE, (void*)ptr);

	if(m_ID != NullID){
		m_IDOwner = true;
		s_WidgetIDMap.insert(std::pair<int,MQWidgetBase*>(m_ID, this));
	}
}

MQDoubleSpinBox::MQDoubleSpinBox(int id) : MQWidgetBase(id)
{
}

MQDoubleSpinBox::~MQDoubleSpinBox()
{
}

double MQDoubleSpinBox::GetPosition()
{
	double value = 0;
	void *ptr[3];
	ptr[0] = "position";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQDoubleSpinBox::SetPosition(double value)
{
	void *ptr[3];
	ptr[0] = "position";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

double MQDoubleSpinBox::GetMin()
{
	double value = 0;
	void *ptr[3];
	ptr[0] = "min";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQDoubleSpinBox::SetMin(double value)
{
	void *ptr[3];
	ptr[0] = "min";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

double MQDoubleSpinBox::GetMax()
{
	double value = 0;
	void *ptr[3];
	ptr[0] = "max";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQDoubleSpinBox::SetMax(double value)
{
	void *ptr[3];
	ptr[0] = "max";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

double MQDoubleSpinBox::GetIncrement()
{
	double value = 0;
	void *ptr[3];
	ptr[0] = "increment";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQDoubleSpinBox::SetIncrement(double value)
{
	void *ptr[3];
	ptr[0] = "increment";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

bool MQDoubleSpinBox::GetExponential()
{
	bool value = false;
	void *ptr[3];
	ptr[0] = "exponential";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQDoubleSpinBox::SetExponential(bool value)
{
	void *ptr[3];
	ptr[0] = "exponential";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

double MQDoubleSpinBox::GetMantissa()
{
	double value = 0.0;
	void *ptr[3];
	ptr[0] = "mantissa";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQDoubleSpinBox::SetMantissa(double value)
{
	void *ptr[3];
	ptr[0] = "mantissa";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

int MQDoubleSpinBox::GetDecimalDigit()
{
	int value = 0;
	void *ptr[3];
	ptr[0] = "decimaldigit";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQDoubleSpinBox::SetDecimalDigit(int value)
{
	void *ptr[3];
	ptr[0] = "decimaldigit";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

bool MQDoubleSpinBox::GetVariableDigit()
{
	bool value = false;
	void *ptr[3];
	ptr[0] = "variabledigit";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQDoubleSpinBox::SetVariableDigit(bool value)
{
	void *ptr[3];
	ptr[0] = "variabledigit";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

int MQDoubleSpinBox::GetAutoDigit()
{
	int value = 0;
	void *ptr[3];
	ptr[0] = "autodigit";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQDoubleSpinBox::SetAutoDigit(int value)
{
	void *ptr[3];
	ptr[0] = "autodigit";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

int MQDoubleSpinBox::GetMaxDecimalDigit()
{
	int value = 0;
	void *ptr[3];
	ptr[0] = "maxdecimaldigit";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQDoubleSpinBox::SetMaxDecimalDigit(int value)
{
	void *ptr[3];
	ptr[0] = "maxdecimaldigit";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

MQDoubleSpinBox::MQDoubleSpinBoxTextAlignment MQDoubleSpinBox::GetAlignment()
{
	char *align_str = NULL;
	void *ptr[3];
	ptr[0] = "alignment";
	ptr[1] = &align_str;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	if(strcmp(align_str, "left") == 0) return ALIGN_LEFT;
	if(strcmp(align_str, "right") == 0) return ALIGN_RIGHT;
	return ALIGN_LEFT;
}

void MQDoubleSpinBox::SetAlignment(MQDoubleSpinBoxTextAlignment value)
{
	void *ptr[3];
	ptr[0] = "alignment";
	switch(value){
	default:
	case ALIGN_LEFT:   ptr[1] = "left"; break;
	case ALIGN_RIGHT:  ptr[1] = "right"; break;
	}
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

int MQDoubleSpinBox::GetVisibleColumn()
{
	int value = 0;
	void *ptr[3];
	ptr[0] = "visiblecolumn";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQDoubleSpinBox::SetVisibleColumn(int value)
{
	void *ptr[3];
	ptr[0] = "visiblecolumn";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

void MQDoubleSpinBox::SetFocus(bool value)
{
	void *ptr[3];
	ptr[0] = "focus";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}


//---------------------------------------------------------------------------
//  class MQSlider
//---------------------------------------------------------------------------
MQSlider::MQSlider() : MQWidgetBase()
{
	void *ptr[5];
	ptr[0] = "type";
	ptr[1] = "slider";
	ptr[2] = "id";
	ptr[3] = &m_ID;
	ptr[4] = NULL;
	MQWidget_Value(NullID, MQWIDGET_CREATE, (void*)ptr);

	if(m_ID != NullID){
		m_IDOwner = true;
		s_WidgetIDMap.insert(std::pair<int,MQWidgetBase*>(m_ID, this));
	}
}

MQSlider::MQSlider(int id) : MQWidgetBase(id)
{
}

MQSlider::~MQSlider()
{
}

double MQSlider::GetPosition()
{
	double value = 0;
	void *ptr[3];
	ptr[0] = "position";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQSlider::SetPosition(double value)
{
	void *ptr[3];
	ptr[0] = "position";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

double MQSlider::GetMin()
{
	double value = 0;
	void *ptr[3];
	ptr[0] = "min";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQSlider::SetMin(double value)
{
	void *ptr[3];
	ptr[0] = "min";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

double MQSlider::GetMax()
{
	double value = 0;
	void *ptr[3];
	ptr[0] = "max";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQSlider::SetMax(double value)
{
	void *ptr[3];
	ptr[0] = "max";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}


//---------------------------------------------------------------------------
//  class MQScrollBar
//---------------------------------------------------------------------------
MQScrollBar::MQScrollBar() : MQWidgetBase()
{
	void *ptr[5];
	ptr[0] = "type";
	ptr[1] = "scrollbar";
	ptr[2] = "id";
	ptr[3] = &m_ID;
	ptr[4] = NULL;
	MQWidget_Value(NullID, MQWIDGET_CREATE, (void*)ptr);

	if(m_ID != NullID){
		m_IDOwner = true;
		s_WidgetIDMap.insert(std::pair<int,MQWidgetBase*>(m_ID, this));
	}
}

MQScrollBar::MQScrollBar(int id) : MQWidgetBase(id)
{
}

MQScrollBar::~MQScrollBar()
{
}

int MQScrollBar::GetPosition()
{
	int value = 0;
	void *ptr[3];
	ptr[0] = "position";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQScrollBar::SetPosition(int value)
{
	void *ptr[3];
	ptr[0] = "position";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

int MQScrollBar::GetMin()
{
	int value = 0;
	void *ptr[3];
	ptr[0] = "min";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQScrollBar::SetMin(int value)
{
	void *ptr[3];
	ptr[0] = "min";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

int MQScrollBar::GetMax()
{
	int value = 0;
	void *ptr[3];
	ptr[0] = "max";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQScrollBar::SetMax(int value)
{
	void *ptr[3];
	ptr[0] = "max";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

int MQScrollBar::GetPage()
{
	int value = 0;
	void *ptr[3];
	ptr[0] = "page";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQScrollBar::SetPage(int value)
{
	void *ptr[3];
	ptr[0] = "page";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

int MQScrollBar::GetIncrement()
{
	int value = 0;
	void *ptr[3];
	ptr[0] = "increment";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQScrollBar::SetIncrement(int value)
{
	void *ptr[3];
	ptr[0] = "increment";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}


//---------------------------------------------------------------------------
//  class MQColorPanel
//---------------------------------------------------------------------------
MQColorPanel::MQColorPanel() : MQWidgetBase()
{
	void *ptr[5];
	ptr[0] = "type";
	ptr[1] = "colorpanel";
	ptr[2] = "id";
	ptr[3] = &m_ID;
	ptr[4] = NULL;
	MQWidget_Value(NullID, MQWIDGET_CREATE, (void*)ptr);

	if(m_ID != NullID){
		m_IDOwner = true;
		s_WidgetIDMap.insert(std::pair<int,MQWidgetBase*>(m_ID, this));
	}
}

MQColorPanel::MQColorPanel(int id) : MQWidgetBase(id)
{
}

MQColorPanel::~MQColorPanel()
{
}

void MQColorPanel::GetColor(int& r, int& g, int& b)
{
	int array[3] = {0,0,0};
	void *ptr[3];
	ptr[0] = "rgb";
	ptr[1] = array;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	r = array[0];
	g = array[1];
	b = array[2];
}

void MQColorPanel::SetColor(int r, int g, int b)
{
	int array[3] = {r,g,b};
	void *ptr[7];
	ptr[0] = "rgb";
	ptr[1] = array;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

void MQColorPanel::GetHSV(double& h, double& s, double& v)
{
	double array[3] = {0,0,0};
	void *ptr[3];
	ptr[0] = "hsv";
	ptr[1] = array;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	h = array[0];
	s = array[1];
	v = array[2];
}

void MQColorPanel::SetHSV(double h, double s, double v)
{
	double array[3] = {h,s,v};
	void *ptr[3];
	ptr[0] = "hsv";
	ptr[1] = array;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}


//---------------------------------------------------------------------------
//  class MQWindowBase
//---------------------------------------------------------------------------
MQWindowBase::MQWindowBase() : MQWidgetBase()
{
}

MQWindowBase::MQWindowBase(int id) : MQWidgetBase(id)
{
}

MQWindowBase::~MQWindowBase()
{
	std::set<MQWidgetBase*>::iterator it;
	for(it = m_CreatedWidgets.begin(); it != m_CreatedWidgets.end(); ){
		delete *it;
		it = m_CreatedWidgets.erase(it);
	}
}

void MQWindowBase::AddChildWindow(MQWindowBase *child)
{
	if(child == NULL) return;

	void *ptr[5];
	int child_id = child->GetID();
	int result = -1;

	ptr[0] = "child";
	ptr[1] = &child_id;
	ptr[2] = "result";
	ptr[3] = &result;
	ptr[4] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_ADD_CHILD_WINDOW, (void*)ptr);
}

void MQWindowBase::RemoveChildWindow(MQWindowBase *child)
{
	if(child == NULL) return;

	void *ptr[5];
	int child_id = child->GetID();
	int result = -1;

	ptr[0] = "child";
	ptr[1] = &child_id;
	ptr[2] = "result";
	ptr[3] = &result;
	ptr[4] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_REMOVE_CHILD_WINDOW, (void*)ptr);
}

MQFrame *MQWindowBase::CreateHorizontalFrame(MQWidgetBase *parent)
{
	MQFrame *p = new MQFrame();
	m_CreatedWidgets.insert(p);
	p->SetAlignment(MQFrame::ALIGN_HORIZONTAL);
	if(parent != NULL){
		parent->AddChild(p);
	}
	return p;
}

MQFrame *MQWindowBase::CreateVerticalFrame(MQWidgetBase *parent)
{
	MQFrame *p = new MQFrame();
	m_CreatedWidgets.insert(p);
	p->SetAlignment(MQFrame::ALIGN_VERTICAL);
	if(parent != NULL){
		parent->AddChild(p);
	}
	return p;
}

MQGroupBox *MQWindowBase::CreateGroupBox(MQWidgetBase *parent, const std::wstring& str)
{
	MQGroupBox *p = new MQGroupBox();
	m_CreatedWidgets.insert(p);
	p->SetText(str);
	if(parent != NULL){
		parent->AddChild(p);
	}
	return p;
}

MQTab *MQWindowBase::CreateTab(MQWidgetBase *parent)
{
	MQTab *p = new MQTab();
	m_CreatedWidgets.insert(p);
	if(parent != NULL){
		parent->AddChild(p);
	}
	return p;
}

MQButton *MQWindowBase::CreateButton(MQWidgetBase *parent, const std::wstring& str)
{
	MQButton *p = new MQButton();
	m_CreatedWidgets.insert(p);
	p->SetText(str);
	if(parent != NULL){
		parent->AddChild(p);
	}
	return p;
}

MQCheckBox *MQWindowBase::CreateCheckBox(MQWidgetBase *parent, const std::wstring& str)
{
	MQCheckBox *p = new MQCheckBox();
	m_CreatedWidgets.insert(p);
	p->SetText(str);
	if(parent != NULL){
		parent->AddChild(p);
	}
	return p;
}

MQRadioButton *MQWindowBase::CreateRadioButton(MQWidgetBase *parent, const std::wstring& str)
{
	MQRadioButton *p = new MQRadioButton();
	m_CreatedWidgets.insert(p);
	p->SetText(str);
	if(parent != NULL){
		parent->AddChild(p);
	}
	return p;
}

MQComboBox *MQWindowBase::CreateComboBox(MQWidgetBase *parent)
{
	MQComboBox *p = new MQComboBox();
	m_CreatedWidgets.insert(p);
	if(parent != NULL){
		parent->AddChild(p);
	}
	return p;
}

MQListBox *MQWindowBase::CreateListBox(MQWidgetBase *parent)
{
	MQListBox *p = new MQListBox();
	m_CreatedWidgets.insert(p);
	if(parent != NULL){
		parent->AddChild(p);
	}
	return p;
}

MQLabel *MQWindowBase::CreateLabel(MQWidgetBase *parent, const std::wstring& str)
{
	MQLabel *p = new MQLabel();
	m_CreatedWidgets.insert(p);
	p->SetText(str);
	if(parent != NULL){
		parent->AddChild(p);
	}
	return p;
}

MQEdit *MQWindowBase::CreateEdit(MQWidgetBase *parent, const std::wstring& str)
{
	MQEdit *p = new MQEdit();
	m_CreatedWidgets.insert(p);
	p->SetText(str);
	if(parent != NULL){
		parent->AddChild(p);
	}
	return p;
}

MQMemo *MQWindowBase::CreateMemo(MQWidgetBase *parent)
{
	MQMemo *p = new MQMemo();
	m_CreatedWidgets.insert(p);
	if(parent != NULL){
		parent->AddChild(p);
	}
	return p;
}

MQSpinBox *MQWindowBase::CreateSpinBox(MQWidgetBase *parent)
{
	MQSpinBox *p = new MQSpinBox();
	m_CreatedWidgets.insert(p);
	if(parent != NULL){
		parent->AddChild(p);
	}
	return p;
}

MQDoubleSpinBox *MQWindowBase::CreateDoubleSpinBox(MQWidgetBase *parent)
{
	MQDoubleSpinBox *p = new MQDoubleSpinBox();
	m_CreatedWidgets.insert(p);
	if(parent != NULL){
		parent->AddChild(p);
	}
	return p;
}

MQSlider *MQWindowBase::CreateSlider(MQWidgetBase *parent)
{
	MQSlider *p = new MQSlider();
	m_CreatedWidgets.insert(p);
	if(parent != NULL){
		parent->AddChild(p);
	}
	return p;
}

MQScrollBar *MQWindowBase::CreateScrollBar(MQWidgetBase *parent)
{
	MQScrollBar *p = new MQScrollBar();
	m_CreatedWidgets.insert(p);
	if(parent != NULL){
		parent->AddChild(p);
	}
	return p;
}

MQColorPanel *MQWindowBase::CreateColorPanel(MQWidgetBase *parent)
{
	MQColorPanel *p = new MQColorPanel();
	m_CreatedWidgets.insert(p);
	if(parent != NULL){
		parent->AddChild(p);
	}
	return p;
}

void MQWindowBase::DeleteWidget(MQWidgetBase *widget)
{
	assert(widget != NULL);
	if(widget == NULL) return;

	m_CreatedWidgets.erase(widget);

	delete widget;
}

std::wstring MQWindowBase::GetTitle()
{
	void *ptr[3];
	unsigned int length = 0;

	ptr[0] = "title.length";
	ptr[1] = &length;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	if(length == 0){
		return std::wstring();
	}

	wchar_t *buf = new wchar_t[length+1];
	ptr[0] = "title";
	ptr[1] = buf;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	std::wstring ret(buf);
	delete[] buf;
	return ret;
}

void MQWindowBase::SetTitle(const std::wstring& text)
{
	void *ptr[3];
	ptr[0] = "title";
	ptr[1] = (void*)text.c_str();
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

//---------------------------------------------------------------------------
//  class MQWindow
//---------------------------------------------------------------------------
MQWindow::MQWindow() : MQWindowBase()
{
	CreateWindowID();
}

MQWindow::MQWindow(int id) : MQWindowBase(id)
{
}

MQWindow::MQWindow(MQWindowBase& parent) : MQWindowBase()
{
	CreateWindowID();
	parent.AddChildWindow(this);
}

MQWindow::~MQWindow()
{
}

void MQWindow::CreateWindowID()
{
	void *ptr[5];
	ptr[0] = "type";
	ptr[1] = "window";
	ptr[2] = "id";
	ptr[3] = &m_ID;
	ptr[4] = NULL;
	MQWidget_Value(NullID, MQWIDGET_CREATE, (void*)ptr);

	if(m_ID != NullID){
		m_IDOwner = true;
		s_WidgetIDMap.insert(std::pair<int,MQWidgetBase*>(m_ID, this));
	}
}

MQWindow MQWindow::GetMainWindow()
{
	int id = GetSystemWidgetID(MQSystemWidget::MainWindow);
	return MQWindow(id);
}

//---------------------------------------------------------------------------
//  class MQDialog
//---------------------------------------------------------------------------
MQDialog::MQDialog() : MQWindowBase()
{
	CreateDialogID();
}

MQDialog::MQDialog(int id) : MQWindowBase(id)
{
}

MQDialog::MQDialog(MQWindowBase& parent) : MQWindowBase()
{
	CreateDialogID();
	parent.AddChildWindow(this);
}

MQDialog::~MQDialog()
{
}

void MQDialog::CreateDialogID()
{
	void *ptr[5];
	ptr[0] = "type";
	ptr[1] = "dialog";
	ptr[2] = "id";
	ptr[3] = &m_ID;
	ptr[4] = NULL;
	MQWidget_Value(NullID, MQWIDGET_CREATE, (void*)ptr);

	if(m_ID != NullID){
		m_IDOwner = true;
		s_WidgetIDMap.insert(std::pair<int,MQWidgetBase*>(m_ID, this));
	}
}

MQDialog::DIALOG_RESULT MQDialog::Execute()
{
	MQDialog::DIALOG_RESULT result = MQDialog::DIALOG_NONE;

	void *ptr[5];
	ptr[0] = "type";
	ptr[1] = "modal";
	ptr[2] = "result";
	ptr[3] = &result;
	ptr[4] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_EXECUTE, (void*)ptr);

	return result;
}

void MQDialog::Close(DIALOG_RESULT code)
{
	void *ptr[5];
	ptr[0] = "type";
	ptr[1] = "close";
	ptr[2] = "code";
	switch(code){
	case DIALOG_OK: ptr[3] = "ok"; break;
	case DIALOG_CANCEL: ptr[3] = "cancel"; break;
	case DIALOG_YES: ptr[3] = "yes"; break;
	case DIALOG_NO: ptr[3] = "no"; break;
	case DIALOG_ALL: ptr[3] = "all"; break;
	default: ptr[3] = "none"; break;
	}
	ptr[4] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_EXECUTE, (void*)ptr);
}

MQDialog::DIALOG_RESULT MQDialog::MessageWarningBox(MQWindowBase& parent, const std::wstring& message, const std::wstring& title)
{
	MQDialog::DIALOG_RESULT result = MQDialog::DIALOG_NONE;
	int parent_id = parent.GetID();

	void *ptr[13];
	ptr[0] = "type";
	ptr[1] = "messagebox";
	ptr[2] = "messagetype";
	ptr[3] = "warning";
	ptr[4] = "parent";
	ptr[5] = &parent_id;
	ptr[6] = "message";
	ptr[7] = (void*)message.c_str();
	ptr[8] = "title";
	ptr[9] = (void*)title.c_str();
	ptr[10] = "result";
	ptr[11] = &result;
	ptr[12] = NULL;
	MQWidget_Value(NullID, MQWIDGET_EXECUTE, (void*)ptr);

	return result;
}

MQDialog::DIALOG_RESULT MQDialog::MessageInformationBox(MQWindowBase& parent, const std::wstring& message, const std::wstring& title)
{
	MQDialog::DIALOG_RESULT result = MQDialog::DIALOG_NONE;
	int parent_id = parent.GetID();

	void *ptr[13];
	ptr[0] = "type";
	ptr[1] = "messagebox";
	ptr[2] = "messagetype";
	ptr[3] = "information";
	ptr[4] = "parent";
	ptr[5] = &parent_id;
	ptr[6] = "message";
	ptr[7] = (void*)message.c_str();
	ptr[8] = "title";
	ptr[9] = (void*)title.c_str();
	ptr[10] = "result";
	ptr[11] = &result;
	ptr[12] = NULL;
	MQWidget_Value(NullID, MQWIDGET_EXECUTE, (void*)ptr);

	return result;
}

MQDialog::DIALOG_RESULT MQDialog::MessageYesNoBox(MQWindowBase& parent, const std::wstring& message, const std::wstring& title)
{
	MQDialog::DIALOG_RESULT result = MQDialog::DIALOG_NONE;
	int parent_id = parent.GetID();

	void *ptr[13];
	ptr[0] = "type";
	ptr[1] = "messagebox";
	ptr[2] = "messagetype";
	ptr[3] = "yesno";
	ptr[4] = "parent";
	ptr[5] = &parent_id;
	ptr[6] = "message";
	ptr[7] = (void*)message.c_str();
	ptr[8] = "title";
	ptr[9] = (void*)title.c_str();
	ptr[10] = "result";
	ptr[11] = &result;
	ptr[12] = NULL;
	MQWidget_Value(NullID, MQWIDGET_EXECUTE, (void*)ptr);

	return result;
}

MQDialog::DIALOG_RESULT MQDialog::MessageYesNoCancelBox(MQWindowBase& parent, const std::wstring& message, const std::wstring& title)
{
	MQDialog::DIALOG_RESULT result = MQDialog::DIALOG_NONE;
	int parent_id = parent.GetID();

	void *ptr[13];
	ptr[0] = "type";
	ptr[1] = "messagebox";
	ptr[2] = "messagetype";
	ptr[3] = "yesnocancel";
	ptr[4] = "parent";
	ptr[5] = &parent_id;
	ptr[6] = "message";
	ptr[7] = (void*)message.c_str();
	ptr[8] = "title";
	ptr[9] = (void*)title.c_str();
	ptr[10] = "result";
	ptr[11] = &result;
	ptr[12] = NULL;
	MQWidget_Value(NullID, MQWIDGET_EXECUTE, (void*)ptr);

	return result;
}

MQDialog::DIALOG_RESULT MQDialog::MessageOkCancelBox(MQWindowBase& parent, const std::wstring& message, const std::wstring& title)
{
	MQDialog::DIALOG_RESULT result = MQDialog::DIALOG_NONE;
	int parent_id = parent.GetID();

	void *ptr[13];
	ptr[0] = "type";
	ptr[1] = "messagebox";
	ptr[2] = "messagetype";
	ptr[3] = "okcancel";
	ptr[4] = "parent";
	ptr[5] = &parent_id;
	ptr[6] = "message";
	ptr[7] = (void*)message.c_str();
	ptr[8] = "title";
	ptr[9] = (void*)title.c_str();
	ptr[10] = "result";
	ptr[11] = &result;
	ptr[12] = NULL;
	MQWidget_Value(NullID, MQWIDGET_EXECUTE, (void*)ptr);

	return result;
}


//---------------------------------------------------------------------------
//  class MQFileDialogBase
//---------------------------------------------------------------------------
MQFileDialogBase::MQFileDialogBase()
{
}

MQFileDialogBase::~MQFileDialogBase()
{
	if(m_ID != NullID){
		//s_WidgetIDMap.erase(m_ID);

		void *ptr[1];
		ptr[0] = NULL;
		MQWidget_Value(m_ID, MQWIDGET_DELETE, (void*)ptr);

		m_ID = NullID;
	}
}

int MQFileDialogBase::AddFilter(const std::wstring& val)
{
	int result = -1;
	void *ptr[7];
	ptr[0] = "addfilter";
	ptr[1] = NULL;
	ptr[2] = "text";
	ptr[3] = (void*)val.c_str();
	ptr[4] = "result";
	ptr[5] = &result;
	ptr[6] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_EDIT, (void*)ptr);
	return result;
}

int MQFileDialogBase::GetFilterCount() const
{
	int value = 0;
	void *ptr[3];

	ptr[0] = "filter.count";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);
	return value;
}

std::wstring MQFileDialogBase::GetFilter(int index) const
{
	void *ptr[5];
	unsigned int length = 0;

	ptr[0] = "index";
	ptr[1] = &index;
	ptr[2] = "filter.text.length";
	ptr[3] = &length;
	ptr[4] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	if(length == 0){
		return std::wstring();
	}

	wchar_t *buf = new wchar_t[length+1];
	ptr[2] = "filter.text";
	ptr[3] = buf;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	std::wstring ret(buf);
	delete[] buf;
	return ret;
}

void MQFileDialogBase::SetFilter(int index, const std::wstring& val)
{
	void *ptr[5];
	ptr[0] = "index";
	ptr[1] = &index;
	ptr[2] = "filter.text";
	ptr[3] = (void*)val.c_str();
	ptr[4] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

void MQFileDialogBase::ClearFilters()
{
	void *ptr[3];
	ptr[0] = "clearfilters";
	ptr[1] = NULL;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_EDIT, (void*)ptr);
}

std::wstring MQFileDialogBase::GetTitle() const
{
	void *ptr[3];
	unsigned int length = 0;

	ptr[0] = "title.length";
	ptr[1] = &length;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	if(length == 0){
		return std::wstring();
	}

	wchar_t *buf = new wchar_t[length+1];
	ptr[0] = "title";
	ptr[1] = buf;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	std::wstring ret(buf);
	delete[] buf;
	return ret;
}

void MQFileDialogBase::SetTitle(const std::wstring& val)
{
	void *ptr[3];
	ptr[0] = "title";
	ptr[1] = (void*)val.c_str();
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

std::wstring MQFileDialogBase::GetFileName() const
{
	void *ptr[3];
	unsigned int length = 0;

	ptr[0] = "filename.length";
	ptr[1] = &length;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	if(length == 0){
		return std::wstring();
	}

	wchar_t *buf = new wchar_t[length+1];
	ptr[0] = "filename";
	ptr[1] = buf;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	std::wstring ret(buf);
	delete[] buf;
	return ret;
}

void MQFileDialogBase::SetFileName(const std::wstring& val)
{
	void *ptr[3];
	ptr[0] = "filename";
	ptr[1] = (void*)val.c_str();
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

std::wstring MQFileDialogBase::GetInitialDir() const
{
	void *ptr[3];
	unsigned int length = 0;

	ptr[0] = "initialdir.length";
	ptr[1] = &length;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	if(length == 0){
		return std::wstring();
	}

	wchar_t *buf = new wchar_t[length+1];
	ptr[0] = "initialdir";
	ptr[1] = buf;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	std::wstring ret(buf);
	delete[] buf;
	return ret;
}

void MQFileDialogBase::SetInitialDir(const std::wstring& val)
{
	void *ptr[3];
	ptr[0] = "initialdir";
	ptr[1] = (void*)val.c_str();
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

std::wstring MQFileDialogBase::GetDefaultExt() const
{
	void *ptr[3];
	unsigned int length = 0;

	ptr[0] = "defaultext.length";
	ptr[1] = &length;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	if(length == 0){
		return std::wstring();
	}

	wchar_t *buf = new wchar_t[length+1];
	ptr[0] = "defaultext";
	ptr[1] = buf;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	std::wstring ret(buf);
	delete[] buf;
	return ret;
}

void MQFileDialogBase::SetDefaultExt(const std::wstring& val)
{
	void *ptr[3];
	ptr[0] = "defaultext";
	ptr[1] = (void*)val.c_str();
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

int MQFileDialogBase::GetFilterIndex() const
{
	int value = 0;
	void *ptr[3];
	ptr[0] = "filterindex";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQFileDialogBase::SetFilterIndex(int val)
{
	void *ptr[3];
	ptr[0] = "filterindex";
	ptr[1] = &val;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

bool MQFileDialogBase::GetNoChangeDir() const
{
	bool value = false;
	void *ptr[3];
	ptr[0] = "nochangedir";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQFileDialogBase::SetNoChangeDir(bool val)
{
	void *ptr[3];
	ptr[0] = "nochangedir";
	ptr[1] = &val;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}


//---------------------------------------------------------------------------
//  class MQOpenFileDialog
//---------------------------------------------------------------------------
MQOpenFileDialog::MQOpenFileDialog(MQWindowBase& parent) : MQFileDialogBase()
{
	int parentID = parent.GetID();

	void *ptr[7];
	ptr[0] = "type";
	ptr[1] = "openfiledialog";
	ptr[2] = "id";
	ptr[3] = &m_ID;
	ptr[4] = "parent";
	ptr[5] = &parentID;
	ptr[6] = NULL;
	MQWidget_Value(NullID, MQWIDGET_CREATE, (void*)ptr);

	if(m_ID != NullID){
		//s_WidgetIDMap.insert(std::pair<int,MQWidgetBase*>(m_ID, this));
	}
}

MQOpenFileDialog::~MQOpenFileDialog()
{
}

bool MQOpenFileDialog::GetFileMustExist() const
{
	bool value = false;
	void *ptr[3];
	ptr[0] = "filemustexist";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQOpenFileDialog::SetFileMustExist(bool val)
{
	void *ptr[3];
	ptr[0] = "filemustexist";
	ptr[1] = &val;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

bool MQOpenFileDialog::GetMultiSelect() const
{
	bool value = false;
	void *ptr[3];
	ptr[0] = "multiselect";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	return value;
}

void MQOpenFileDialog::SetMultiSelect(bool val)
{
	void *ptr[3];
	ptr[0] = "multiselect";
	ptr[1] = &val;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_SET, (void*)ptr);
}

bool MQOpenFileDialog::Execute()
{
	bool result = false;

	void *ptr[5];
	ptr[0] = "type";
	ptr[1] = "fileopen";
	ptr[2] = "result";
	ptr[3] = &result;
	ptr[4] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_EXECUTE, (void*)ptr);

	return result;
}

int MQOpenFileDialog::GetFileNamesCount() const
{
	int value = 0;
	void *ptr[3];

	ptr[0] = "filenames.count";
	ptr[1] = &value;
	ptr[2] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);
	return value;
}

std::wstring MQOpenFileDialog::GetFileNames(int index) const
{
	void *ptr[5];
	unsigned int length = 0;

	ptr[0] = "index";
	ptr[1] = &index;
	ptr[2] = "filenames.length";
	ptr[3] = &length;
	ptr[4] = NULL;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	if(length == 0){
		return std::wstring();
	}

	wchar_t *buf = new wchar_t[length+1];
	ptr[2] = "filenames";
	ptr[3] = buf;
	MQWidget_Value(m_ID, MQWIDGET_GET, (void*)ptr);

	std::wstring ret(buf);
	delete[] buf;
	return ret;
}


//---------------------------------------------------------------------------
//  class MQSaveFileDialog
//---------------------------------------------------------------------------
MQSaveFileDialog::MQSaveFileDialog(MQWindowBase& parent) : MQFileDialogBase()
{
	int parentID = parent.GetID();

	void *ptr[7];
	ptr[0] = "type";
	ptr[1] = "openfiledialog";
	ptr[2] = "id";
	ptr[3] = &m_ID;
	ptr[4] = "parent";
	ptr[5] = &parentID;
	ptr[6] = NULL;
	MQWidget_Value(NullID, MQWIDGET_CREATE, (void*)ptr);

	if(m_ID != NullID){
		//s_WidgetIDMap.insert(std::pair<int,MQWidgetBase*>(m_ID, this));
	}
}

MQSaveFileDialog::~MQSaveFileDialog()
{
}



void *ExtractEventOption(void *option, const char *name)
{
	if (option != NULL){
		void **array = (void**)option;
		for (int i=0; array[i]!=NULL; i+=2){
			if (strcmp((char*)array[i], name) == 0)
				return array[i+1];
		}
	}
	return NULL;
}

BOOL ExtractEventOptionBool(void *option, const char *name, BOOL defval)
{
	BOOL *ptr = (BOOL*)ExtractEventOption(option, name);
	return (ptr != NULL) ? *ptr : defval;
}

int ExtractEventOptionInt(void *option, const char *name, int defval)
{
	int *ptr = (int*)ExtractEventOption(option, name);
	return (ptr != NULL) ? *ptr : defval;
}

float ExtractEventOptionFloat(void *option, const char *name, float defval)
{
	float *ptr = (float*)ExtractEventOption(option, name);
	return (ptr != NULL) ? *ptr : defval;
}

